
# DO NOT EDIT THIS FILE -- AUTOGENERATED BY PANTS
# Target: //:liga-easyocr

from setuptools import setup

setup(**{
    'install_requires': (
        'easyocr',
        'ligavision',
    ),
    'name': 'liga-easyocr',
    'namespace_packages': (
    ),
    'package_data': {
    },
    'packages': (
        'liga.easyocr',
        'liga.easyocr.models',
    ),
    'python_requires': '<3.10,>=3.8',
    'version': '0.2.0.dev0',
})
